@@ -228,7 +228,7 @@ pending_find_match(struct replay_runtime* runtime, struct entry** entry,
  * @param runtime: runtime.
  * @param entry: if true, the entry that matches is returned.
  * @param pend: if true, the outgoing message that matches is returned.
- * return: true if pending query matches the now event.
+ * @return: true if pending query matches the now event.
  */
 static int 
 pending_matches_range(struct replay_runtime* runtime, 
@@ -1007,8 +1007,8 @@ pending_tcp_query(struct outside_network* outnet, ldns_buffer* packet,
 struct serviced_query* outnet_serviced_query(struct outside_network* outnet,
         uint8_t* qname, size_t qnamelen, uint16_t qtype, uint16_t qclass,
 	uint16_t flags, int dnssec, int ATTR_UNUSED(want_dnssec),
-	struct sockaddr_storage* addr, socklen_t addrlen, 
-	comm_point_callback_t* callback, void* callback_arg, 
+	int ATTR_UNUSED(tcp_upstream), struct sockaddr_storage* addr,
+	socklen_t addrlen, comm_point_callback_t* callback, void* callback_arg, 
 	ldns_buffer* ATTR_UNUSED(buff), int (*arg_compare)(void*,void*))
 {
 	struct replay_runtime* runtime = (struct replay_runtime*)outnet->base;
@@ -1119,14 +1119,6 @@ void listening_ports_free(struct listen_port* list)
 	free(list);
 }
 
-void listen_pushback(struct listen_dnsport* ATTR_UNUSED(listen))
-{
-}
-
-void listen_resume(struct listen_dnsport* ATTR_UNUSED(listen))
-{
-}
-
 struct comm_point* comm_point_create_local(struct comm_base* ATTR_UNUSED(base),
         int ATTR_UNUSED(fd), size_t ATTR_UNUSED(bufsize),
         comm_point_callback_t* ATTR_UNUSED(callback), 
