@@ -82,24 +82,6 @@ struct module_env {
 	struct key_cache* key_cache;
 
 	/* --- services --- */
-	/** 
-	 * Direct access to the network, this packet gets sent to destination.
-	 * Send DNS query to server. operate() should return with wait_reply.
-	 * Later on a callback will cause operate() to be called with event
-	 * timeout or reply. Replied packet is then in the query buffer.
-	 * @param pkt: packet to send.
-	 * @param addr: where to.
-	 * @param addrlen: length of addr.
-	 * @param timeout: seconds to wait until timeout.
-	 * @param q: wich query state to reactivate upon return.
-	 * @param use_tcp: set to true to send over TCP. 0 for UDP.
-	 * @return: false on failure (memory or socket related). no query was
-	 *	sent.
-	 */
-	int (*send_packet)(ldns_buffer* pkt, struct sockaddr_storage* addr,
-		socklen_t addrlen, int timeout, struct module_qstate* q,
-		int use_tcp);
-
 	/** 
 	 * Send serviced DNS query to server. UDP/TCP and EDNS is handled.
 	 * operate() should return with wait_reply. Later on a callback 
@@ -114,6 +96,8 @@ struct module_env {
 	 * @param dnssec: if set, EDNS record will have bits set.
 	 *	If EDNS_DO bit is set, DO bit is set in EDNS records.
 	 *	If BIT_CD is set, CD bit is set in queries with EDNS records.
+	 * @param want_dnssec: if set, the validator wants DNSSEC.  Without
+	 * 	EDNS, the answer is likely to be useless for this domain.
 	 * @param addr: where to.
 	 * @param addrlen: length of addr.
 	 * @param q: wich query state to reactivate upon return.
@@ -124,8 +108,8 @@ struct module_env {
 	 */
 	struct outbound_entry* (*send_query)(uint8_t* qname, size_t qnamelen, 
 		uint16_t qtype, uint16_t qclass, uint16_t flags, int dnssec, 
-		struct sockaddr_storage* addr, socklen_t addrlen, 
-		struct module_qstate* q);
+		int want_dnssec, struct sockaddr_storage* addr, 
+		socklen_t addrlen, struct module_qstate* q);
 
 	/**
 	 * Detach-subqueries.
